#include "TM1637.h"   //Подключение библиотеки TM1637.h

//---------------------------------- 
bool its_debug = false;  //Режим отладки (если =TRUE, то используется Serial)
//---------------------------------- 
//---------------------------------- 
//Датчик MQ7
#define PIN_MQ7 A5  //пин для подключения MQ-7
int v_mq7 = 0;      //значение, "пришедшее" от датчика MQ-7
//---------------------------------- 
//---------------------------------- 
//Зуммер (buzzer, пищалка)
#define PIN_Buzzer 5     //пин для подключения "пищалки"
int Buzzer_Freq = 1000;  //частота звучания "пищалки"
int Buzzer_Delay = 500;  //длительность звучания "пищалки"
//---------------------------------- 
//---------------------------------- 
//Светодиоды
#define PIN_LED_R  12   //пин для подключения "красного светодиода"
#define PIN_LED_G  11   //пин для подключения "зеленого светодиода"
#define PIN_LED_IN 13   //встроенный светодиод
//---------------------------------- 
//---------------------------------- 
//Дисплей 3642BH
#define CLK 2                //вывод CLK
#define DIO 3                //вывод DIO
#define BRIGHT_TYPICAL  7    //яркость дисплея от 0 до 7
TM1637 tm1637(CLK,DIO);      //Создается объект tm1637
int8_t NumTab[] = {0,0,0,0}; //Создается массив данных, 0-9 это 0-9, от 10 - 15 это A-F. 
//---------------------------------- 
//---------------------------------- 
//Тумблеры (граница допустимых значений)
#define BTN_0 10     //тумблер S0; разряд-0 числа (в двоичном исчислении)
#define BTN_1 9      //тумблер S1; разряд-1 числа (в двоичном исчислении)
#define BTN_2 8      //тумблер S2; разряд-2 числа (в двоичном исчислении)
#define BTN_3 7      //тумблер S3; разряд-3 числа (в двоичном исчислении)
#define BTN_CTRL 4   //тумблер S; "управляющий" тумблер
                     //(HIGH - режим настройки границы; LOW - режим измерения)
int v_ctrl = 0;      //состояние тумблера S; управляюшщий "сигнал"
                     //(HIGH - режим настройки границы; LOW - режим измерения)
int v_crit = 0;        //значение границы (допустимых значений датчика MQ7)
int v_crit_min = 25;   //минимально возможное значение границы (допустимых значений датчика MQ7),
                       //если все тумблеры в положении "0"
int v_crit_delta = 10; //множитель к настроечному значению границы (допустимых значений датчика MQ7)
//---------------------------------- 
//---------------------------------- 
int DelayBetweenCycle    = 1000; //задержка между циклами опроса датчика MQ7 (миллисек)
int CountSecForWaitReady = 10;   //Заданное кол-во СЕКУНД ожидания стабилизации показаний MQ7
                                 //при включении питания 
int iSecForWaitReady    = 0;     //Счетчик СЕКУНД ожидания стабилизации показаний MQ7
bool DevReady           = false; //Готовность датчика MQ7 к работе                              
//---------------------------------- 

//###################################################
void setup() 
{
  //---------------------------------
  //Режим пинов тумблеров
  pinMode(BTN_0, INPUT_PULLUP);
  pinMode(BTN_1, INPUT_PULLUP);
  pinMode(BTN_2, INPUT_PULLUP);
  pinMode(BTN_3, INPUT_PULLUP);
  pinMode(BTN_CTRL, INPUT_PULLUP);
  //---------------------------------
  //---------------------------------
  //Режим пина зуммера
  pinMode(PIN_Buzzer, OUTPUT);
  //---------------------------------
  //---------------------------------
  //Светодиоды
  pinMode(PIN_LED_IN, OUTPUT);  // Встроенный светодиод. Установка режима - выход
  pinMode(PIN_LED_R, OUTPUT);   // Красный светодиод. Установка режима - выход
  pinMode(PIN_LED_G, OUTPUT);   // Зеленый светодиод. Установка режима - выход
  //...
  //"Мигнуть" светодиодами при включении
  digitalWrite(PIN_LED_IN, LOW);
  digitalWrite(PIN_LED_R, HIGH);
  digitalWrite(PIN_LED_G, HIGH);
  delay(500);
  digitalWrite(PIN_LED_R, LOW);
  digitalWrite(PIN_LED_G, LOW);
  //...
  //---------------------------------
  //---------------------------------
  //Режим пина MQ7
  pinMode(PIN_MQ7, INPUT);
  //---------------------------------
  //---------------------------------
  //Дисплей 3642BH
  tm1637.init();                    // Инициализируется дисплей 
  tm1637.set(BRIGHT_TYPICAL);       // Устанавливается яркость дисплея
  NumTab[3]=0;
  NumTab[2]=0;
  NumTab[1]=0;
  NumTab[0]=0;
  tm1637.point(false);    // Выключить двоеточие 
  tm1637.display(NumTab); //Вывод массива на дисплей 
  //---------------------------------
  //---------------------------------
  if (its_debug == true) //режим отладки включен
  {
     Serial.begin(9600);
     Serial.println("================");  
  }
  //---------------------------------
}
//###################################################

//###################################################
void loop() 
{
  //=========================
  if (DevReady == false)
  {
     //Ожидание стабилизации показаний MQ7 заданное кол-во секунд
     iSecForWaitReady = iSecForWaitReady + 1; //Счетчик СЕКУНД
     //..................... 
     IntConvertForDisplay(iSecForWaitReady); //Конвертация v_crit в массив NumTab[] 
                                             //для визуализации на дисплее 3642BH
     tm1637.point(true); //Включить двоеточие на 3642BH
     tm1637.display(NumTab); //Вывод массива на дисплей 
     digitalWrite(PIN_LED_R, HIGH); //Зажечь красный светодиод 
     digitalWrite(PIN_LED_G, HIGH); //Зажечь зеленый светодиод 
     delay(500);
     //..................... 

     if (iSecForWaitReady >= CountSecForWaitReady) //Время ожидания исчерпано
     { 
        DevReady = true; //считается, что датчик MQ7 готов к работе
        //....................
        //"обнуляем" дисплей 3642BH
        NumTab[3]=0;
        NumTab[2]=0;
        NumTab[1]=0;
        NumTab[0]=0;    
        //....................
     }
     
     //..................... 
     tm1637.point(false); //Выключить двоеточие на 3642BH
     tm1637.display(NumTab); //Вывод массива на дисплей 
     digitalWrite(PIN_LED_R, LOW); //Погасить красный светодиод 
     digitalWrite(PIN_LED_G, LOW); //Погасить зеленый светодиод 
     delay(500);
     //..................... 

     //.......................
     if (its_debug == true)  //режим отладки включен
     {
        Serial.println("..................");
        Serial.print("Wait: iSecForWaitReady=");
        Serial.print(iSecForWaitReady);
        Serial.print(" ; CountSecForWaitReady=");
        Serial.print(CountSecForWaitReady);
        Serial.print("  ;  DevReady=");
        Serial.print(DevReady);
        Serial.println("");
     }
     //.......................
  }
  //=========================
  //=========================
  if (DevReady == true)
  {
     //Работа Устройства
     //.....................
     //Считывание состояния тумблера S (режим работы Устройства)
     v_ctrl = digitalRead(BTN_CTRL);  
     //.....................
     //.....................
     //Считывание состояния тумблеров S0..S3 
     //и вычисление границы допустимых значений датчика MQ7
     v_crit = v_crit_delta * PinsToInt(BTN_3, BTN_2, BTN_1, BTN_0) + v_crit_min;
     //.....................
     //.....................
     //Считывание значений датчика MQ7
     v_mq7 = analogRead(PIN_MQ7); 
     //.....................
   
     if (v_ctrl == LOW)  //Режим измерения
     {
        //......................
        digitalWrite(PIN_LED_G, LOW); //Погасить зеленый светодиод  
        IntConvertForDisplay(v_mq7);  //Конвертация v_crit в массив NumTab[] 
                                      //для визуализации на дисплее 3642BH
        tm1637.point(false);          //Выключить двоеточие 
        tm1637.display(NumTab);       //Вывод массива на дисплей 
        //......................
   
        if (v_mq7 >= v_crit) //Значения от MQ7 превысили границу допустимости
        { 
           //.......................
           //Включить "сигнализацию"
           digitalWrite(PIN_LED_IN, HIGH); //Зажечь встроенный светодиод
           digitalWrite(PIN_LED_R, HIGH);  //Зажечь красный светодиод
           tone(PIN_Buzzer, Buzzer_Freq);  //Включить звуковой сигнал
           //.......................
        }
        else //Значения от MQ7 в пределах нормы
        {
           //.......................
           //Отключить "сигнализацию"
           digitalWrite(PIN_LED_IN, LOW); //Погасить встроенный светодиод
           digitalWrite(PIN_LED_R, LOW);  //Погасить красный светодиод
           noTone(PIN_Buzzer);            //Отключить звуковой сигнал 
           //.......................
        }
     }
     else //Режим настройки границы допустимых значений датчика MQ7
     {   
        //.......................
        digitalWrite(PIN_LED_IN, LOW); //Погасить встроенный светодиод
        digitalWrite(PIN_LED_R, LOW);  //Погасить красный светодиод
        digitalWrite(PIN_LED_G, HIGH); //Зажечь зеленый светодиод 
        //.......................
        //.......................
        if (its_debug == true)  //режим отладки включен
        {
           Serial.println("-----------------------------");
           Serial.print("Sets. v_crit = ");
           Serial.print(v_crit);
           Serial.println("");
        }
        //.......................
        //.......................
        IntConvertForDisplay(v_crit); //Конвертация v_crit в массив NumTab[] 
                                      //для визуализации на дисплее 3642BH
        tm1637.point(false);          //Выключение двоеточия
        tm1637.display(NumTab);       //Вывод массива NumTab[] на дисплей 3642BH
        //.......................
        //.......................
        if (its_debug == true) //режим отладки включен
        {
           Serial.println("-----------------------------");
        }
        //.......................
     }
     //.......................
     //задержка между циклами опроса датчика MQ7
     if (DelayBetweenCycle>0) { delay(DelayBetweenCycle); } 
     //.......................
  }
}
//###################################################

//###################################################
//Конвертация целого числа (входной параметр) в массив NumTab[] 
//для визуализации на дисплее 3642BH
bool IntConvertForDisplay(int v)
{ 
  bool res = false;
  //...........................
  //защита от некорректных значений входного параметра
  if (v < 0) { v = 0; }
  if (v > 9999) { v = 9999; }
  //...........................
  //...........................
  int n1 = 0;
  int n2 = 0;
  int n3 = 0;
  int n4 = 0;
  double x = 0.0;
  //...........................
  //=======================
  //Конвертация
  //...........................
  //"тысячи"
  x = v/1000;
  n4 = floor(x);
  //...........................
  //...........................
  // "сотни"
  x = (v - n4*1000);
  x = x/100;
  n3 = floor(x);
  //...........................
  //...........................
  //"десятки"
  x = (v - n4*1000 - n3*100);
  x = x/10;
  n2 = floor(x);
  //...........................
  //...........................
  //"единицы"
  n1 = (v - n4*1000 - n3*100 - n2*10);
  //...........................
  //...........................
  if (its_debug == true)  //режим отладки включен
  {
  Serial.print("Display. v= ");
  Serial.print(v);
  Serial.print(" ->  ");
  Serial.print(n4);
  Serial.print(" ");
  Serial.print(n3);
  Serial.print(" ");
  Serial.print(n2);
  Serial.print(" ");
  Serial.print(n1);
  Serial.print(" ");
  Serial.println("");
  }
  //...........................
  //...........................
  //Заполнение массива NumTab[]
  //для визуализации на дисплее 3642BH
  NumTab[3]=n1;
  NumTab[2]=n2;
  NumTab[1]=n3;
  NumTab[0]=n4;
  //...........................
  res = true;
  return res;
}
//###################################################

//###################################################
//Конвертация входных значений v0..v3 (двоичное представление числа)
//в десятичное
int PinsToInt0(int v3=0, int v2=0, int v1=0, int v0=0)
{
  return v0 + 2*v1 + 4*v2 + 8*v3;
}
//###################################################

//###################################################
//Считывание значений пинов (на которых "висят" тумблеры S0..S3)
//и конвертация их (из двоичного представления числа в десятичное)
int PinsToInt(int nPin3, int nPin2, int nPin1, int nPin0)
{
  int res = 0;
  int v0 = 0;
  int v1 = 0;
  int v2 = 0;
  int v3 = 0;
  if (nPin0>1) { v0 = digitalRead(nPin0); }
  if (nPin1>1) { v1 = digitalRead(nPin1); }
  if (nPin2>1) { v2 = digitalRead(nPin2); }
  if (nPin3>1) { v3 = digitalRead(nPin3); }
  res = PinsToInt0(v3, v2, v1, v0);  
  return res;
}
//###################################################
